//--------------------------------------------------------------//
// BrushedMetal
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//

float4x4 mWorld	: WORLD;
float4x4 inv_view_matrix : ViewInverse;
float4x4 texture_space_matrix;
float4x4 view_proj_matrix : ViewProjection;

float4 light2Position	= float4( -500.00, 500.00, 1.00, 1.00 );
float4 light1Position	= float4( 300.00, 500.00, 1.00, 1.00 );
float4 vRingScale		= float4( 0.50, 0.25, 0.75, 1.00 );
float4 vRingCenter		= float4( -1.50, -20.00, 26.00, 1.00 );
float fAmbient			= float( 0.10 );
float4 ambient			= float4( 0.13, 0.16, 0.13, 1.00 );
float4 ringAmbientColor = float4( 0.23, 0.25, 0.20, 1.00 );
float4 ringColor		= float4( 0.93, 0.94, 1.00, 1.00 );
float4 lightColor		= float4( 1.00, 0.94, 0.84, 1.00 );

texture gradientTex;
texture strandTex;

sampler ring_gradient = sampler_state
{
   Texture = <gradientTex>;
   ADDRESSU = MIRROR;
   ADDRESSV = MIRROR;
   MINFILTER = ANISOTROPIC;
   MIPFILTER = LINEAR;
   MIPMAPLODBIAS = 0.250000;
};

sampler strand = sampler_state
{
   Texture = <strandTex>;
   ADDRESSU = CLAMP;
   ADDRESSV = CLAMP;
   MAGFILTER = LINEAR;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
};

struct StrandPair
{
   float Diffuse;
   float Specular;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
	float4 Pos		: POSITION;
	float4 Normal	: NORMAL;
};

struct VS_OUTPUT
{
   float4 ProjPos  : POSITION;
   float4 Normal   : COLOR0;      // Has ambient in W
   float3 View     : TEXCOORD0;
   float3 DirAniso : COLOR1;
   float4 TexPos   : TEXCOORD1;   // Has light intensity in W
   float3 Light1   : TEXCOORD2;
   float3 Light2   : TEXCOORD3;
};

//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------

VS_OUTPUT VtxShader_BrushedMetal( VS_INPUT In )
{
  VS_OUTPUT Out = (VS_OUTPUT)0;

   // Texture space coordinates:
   float3 texPos      = ((float3)In.Pos) - vRingCenter;
   float3 texSpacePos = texPos * vRingScale + float3(0, -40, -40);

   texSpacePos = mul(texSpacePos, (float3x3)texture_space_matrix );

   Out.TexPos.xyz = texSpacePos;

   // Ambient lighting: 
   Out.TexPos.w = ambient.x * 5;

   // Compute direction of anisotropy
   float3 dirAniso = cross(In.Normal, normalize(texPos));

   Out.ProjPos  = mul( view_proj_matrix, In.Pos );

   // Output the normal vector ( w component contains ambient factor ) 
   Out.Normal   = float4( In.Normal * .5 + (float3).5, fAmbient );

   // Propagate direction of anisotropy:
   Out.DirAniso = dirAniso * 0.5 +(float3)0.5;

   // Compute camera position:
   float4 vCameraPosition = mul( inv_view_matrix, float4(0,0,0,1) );

   // Calculate view vector:
   Out.View = normalize( vCameraPosition - mul( mWorld, In.Pos ) );
   
   // Compute light direction vectors: 
   light1Position = mul( inv_view_matrix, light1Position );
   light2Position = mul( inv_view_matrix, light2Position );

   Out.Light1   = normalize(light1Position - In.Pos );
   Out.Light2   = normalize(light2Position - In.Pos );

   return Out;
} 

//----------------------------------------------------------------------------------------------//
// Pixel shader input struct                                                                    //
//----------------------------------------------------------------------------------------------//
struct PS_INPUT
{
   float4 ProjPos  : POSITION;    
   float4 Normal   : COLOR0;      // Normal vector, w component stores ambient contribution coefficient
   float3 View     : TEXCOORD0;   // View vector
   float3 DirAniso : COLOR1;      // Direction of anisotropy
   float4 TexPos   : TEXCOORD1;   // Lookup texture coordinate, w component stores light intensity coefficient
   float3 Light1   : TEXCOORD2;   // Direction vector for light 1
   float3 Light2   : TEXCOORD3;   // Direction vector for light 2
};

//----------------------------------------------------------------------------------------------//
StrandPair StrandLight( float3 normal, float3 light, float3 view, float3 dirAniso )
{
   StrandPair o;

   float  LdA        = dot( light, dirAniso );
   float  VdA        = dot( view,  dirAniso );
   float2 fnLookup   = tex2D( strand, float2( LdA, VdA ) * 0.5 + (float2)0.5 );
   float  spec       = fnLookup.y * fnLookup.y;
   float  diff       = fnLookup.x; 
   float  selfShadow = saturate( dot( normal, light ) );

   o.Diffuse  = diff * selfShadow;
   o.Specular = spec * selfShadow;
   return o;
}


//----------------------------------------------------------------------------------------------//
// Pixel shader                                                                                 //
//----------------------------------------------------------------------------------------------//
float4 PixShader_BrushedMetal( PS_INPUT In ) : COLOR
{
   // Extract components from inputs 
   float3 dirAniso = In.DirAniso * 2 - (float3)1;
   float3 normal   = In.Normal * 2 - (float3)1;
   float3 view     = (float3) In.View;
   float3 light[2] = { (float3) In.Light1, (float3)In.Light2 };
   float  shadow   = In.TexPos.w;
   float  ambient  = In.Normal.w;

   // Strand Lighting //
   float3   color = 0;
   for (int l = 0; l < 2; l++)
   {
      StrandPair strand = StrandLight(normal, light[l], view, dirAniso);
      color += (strand.Diffuse + strand.Specular) * lightColor;
   }

   float3 ringNewColor = tex1D(ring_gradient, length(In.TexPos)) * ringColor + ringAmbientColor;

   color = (color*shadow + ambient) * ringNewColor;
   
   return float4( color, 1);

}

//---------------------------------------------------------------------------------------

technique brushedMetal
{
   pass Single_Pass
   {
      VertexShader = compile vs_2_0 VtxShader_BrushedMetal();
      PixelShader  = compile ps_2_0 PixShader_BrushedMetal();
   }

}

//---------------------------------------------------------------------------------------