//--------------------------------------------------------------//
// Normal Map & Environment Map
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

// transformation matrices
float4x4 mtxWorld	: WORLD;  
float4x4 mtxView	: VIEW;
float4x4 mtxProj	: PROJECTION;
float4   gLight; 
float3   vecView;

texture baseTex;
texture normalTex;
texture reflectTex;

sampler2D baseTex0 = sampler_state
{
	Texture = <baseTex>;
	MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler2D normalTex0 = sampler_state
{
	Texture = <normalTex>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

samplerCUBE reflectTex0 = sampler_state
{
	Texture = <reflectTex>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
	float3 Pos       : POSITION;
	float3 Normal    : NORMAL;
	float4 Diffuse   : COLOR0;
	float4 Specular  : COLOR1;	
	float2 TexUV0	 : TEXCOORD0;
//	float2 TexUV1    : TEXCOORD1;
	float3 Tangent	 : TANGENT;
	float3 Binormal  : BINORMAL;
};

struct VS_OUTPUT
{
    float4 Pos        : POSITION;  
    float2 TexUV0     : TEXCOORD0;
    float3 LightingVec: TEXCOORD1;
    float3 ViewVector : TEXCOORD2;
};


//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------

VS_OUTPUT VtxShader_NormalMap( VS_INPUT In )
{
    VS_OUTPUT Out;
	
	// position xform
	In.Pos   = mul( float4( In.Pos, 1 ), ( float4x3 )mtxWorld );
	//float3 tmpPos = In.Pos;
	In.Pos   = mul( float4( In.Pos, 1 ), ( float4x3 )mtxView );
	Out.Pos  = mul( float4( In.Pos, 1 ), mtxProj );
	
	In.Normal   = mul( In.Normal,   ( float3x3 )mtxWorld );	
	In.Tangent  = mul( In.Tangent,  ( float3x3 )mtxWorld );	
	In.Binormal = mul( In.Binormal, ( float3x3 )mtxWorld );	
	
	float3x3 mtxTan = float3x3( In.Tangent,In.Binormal,In.Normal );	
	
	float3 tanLight = mul( mtxTan,normalize( gLight ) ); // Transform Light into Tangent space	
	tanLight = normalize( tanLight );
	
	Out.ViewVector = normalize( Out.Pos - vecView );
	Out.ViewVector = mul( mtxTan, Out.ViewVector );


	// Pass-thru texc coordinates
	Out.TexUV0 = In.TexUV0;
	Out.LightingVec = tanLight;	
		
    return Out;    
}

//--------------------------------------------------------------------------------------
// Pixel shader
//--------------------------------------------------------------------------------------

float4 PixShader_NormalMapEnvironment( VS_OUTPUT In ) : COLOR
{ 
    float4 baseColour	= tex2D( baseTex0,  In.TexUV0 );
    float4 normal		= tex2D( normalTex0,In.TexUV0 );
    
    baseColour.r = 0.0f;
    //baseColour.g = 0.0f;
	baseColour.b = 0.05f;
		
    baseColour = normalize( baseColour );
 	float3 N   = normalize( ( normal.rgb * 2.0f ) - 1.0f );  	    
	float3 V   = normalize( In.ViewVector );    
	float3 L   = normalize( In.LightingVec );		
 
	// Specular lighting ----
	float3 R = reflect( -L, N );
	R = normalize( R );    
    
	float dpS = dot( V,R );
	dpS = max( 0, dpS );
	dpS = pow( dpS, 20.0f );	    
	// End Spec -------------
    
    // Diffuse lighting -----
	float dp = dot( -L, N );
	dp = max( 0, dp );
	// End Diffuse ----------
	
	//float dp = saturate( dot( -L, N ) );
	
	// Reflection ------------
	float3 refVec = reflect( -V, N );
	refVec = normalize( refVec );
	float3 refCol = texCUBE( reflectTex0, refVec );
	// End Reflection --------
			
	return float4(refCol,1)*0.10f + baseColour * dp + dpS * 1.0f;
}

float4 PixShader_NormalMap( VS_OUTPUT In ) : COLOR
{ 
    float4 baseColour	= tex2D( baseTex0,  In.TexUV0 );
    float4 normal		= tex2D( normalTex0,In.TexUV0 );
    		
    baseColour = normalize( baseColour );
 	float3 N   = normalize( ( normal.rgb * 2.0f ) - 1.0f );  	    
	float3 V   = normalize( In.ViewVector );    
	float3 L   = normalize( In.LightingVec );		
 
	// Specular lighting ----
	float3 R = reflect( -L, N );
	R = normalize( R );    
    
	float dpS = dot( V,R );
	dpS = max( 0, dpS );
	dpS = pow( dpS, 6.0f );	    
	// End Spec -------------
    
    // Diffuse lighting -----
	float dp = dot( -L, N );
	dp = max( 0, dp );
	// End Diffuse ----------
					
	return baseColour * dp + dpS * 0.1f;
}

//---------------------------------------------------------------------------------------

technique NormalMapEffect_With_Environment
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader_NormalMap( );
		PixelShader  = compile ps_2_0 PixShader_NormalMapEnvironment( );          
    }
}

technique NormalMapEffect
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader_NormalMap( );
		PixelShader  = compile ps_2_0 PixShader_NormalMap( );          
    }
}

//---------------------------------------------------------------------------------------